/* This file is part of IPACCT
   Copyright (c) Sergey Poznyakoff

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* Generic singly-linked lists */
/* $Id: list.c,v 1.4 2003/04/08 18:22:52 gray Exp $ */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include <stdlib.h>
#include <ipacct.h>
#include <syslog.h>

struct list_data {
	struct list_data *next;
	char data[1];
};

struct _list {
	 struct list_data *head, *tail;
};

#define ELSIZE(s) (sizeof(struct list_data) - 1 + (s))

static list_t * _list_alloc_internal(size_t size);

struct list_data *
_list_alloc_data(size)
	size_t size;
{
	struct list_data *p;

	p = malloc(ELSIZE(size));
	if (!p) {
		syslog(LOG_CRIT, "not enough memory: exiting");
		abort();
	}
	p->next = NULL;
	return p;
}

void
list_create(listp)
	list_t *listp;
{
	*listp = malloc(sizeof(**listp));
	if (!*listp) {
		syslog(LOG_CRIT, "not enough memory: exiting");
		abort();
	}
	(*listp)->head = (*listp)->tail = NULL;
}

void *
list_alloc(listp, data, size)
	list_t *listp;
	void *data;
	size_t size;
{
	struct list_data *dp;
	list_t list;

	if (!*listp)
		list_create(listp);
	list = *listp;
	dp = _list_alloc_data(size);
	if (!list->tail)
		list->head = dp;
	else
		list->tail->next = dp;
	list->tail = dp;
	memcpy(dp->data, data, size);
	return dp->data;
}


void
list_free(listp)
	list_t *listp;
{
	struct list_data *this, *next;

	if (!*listp)
		return;
	this = (*listp)->head;
	while (this) {
		next = this->next;
		free(this);
		this = next;
	}
	free(*listp);
	*listp = NULL;
}

void
list_iterate(list, fun, data)
	list_t list;
	int (*fun)(void *, void *);
	void *data;
{
	struct list_data *this, *next;

	if (!list)
		return;
	for (this = list->head; this; this = this->next)
		if ((*fun)(this->data, data))
			break;
}
	
